package AI::NeuralNet::FastSOM::Torus;

use strict;
use warnings;

use AI::NeuralNet::FastSOM::Rect;
our @ISA = qw/AI::NeuralNet::FastSOM::Rect/;

our $VERSION = '0.09';

1;

__END__

=pod

=head1 NAME

AI::NeuralNet::FastSOM::Torus - Perl extension for fast Kohonen Maps (torus topology)

=head1 SYNOPSIS

See L<AI::NeuralNet::SOM::Rect> for details.

=head1 SEE ALSO

L<AI::NeuralNet::FastSOM::Rect>

=head1 AUTHOR

Rick Myers, E<lt>jrm@cpan.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009 by Rick Myers

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.0 or,
at your option, any later version of Perl 5 you may have available.

=cut

